package main

import androidx.compose.desktop.ui.tooling.preview.Preview
import androidx.compose.foundation.layout.*
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowBack
import androidx.compose.material.icons.filled.Check
import androidx.compose.material.icons.filled.Close
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.dp
import androidx.compose.ui.window.*
import kotlinx.serialization.encodeToString
import kotlinx.serialization.json.Json
import java.awt.FileDialog
import java.awt.Frame
import java.io.File
import java.io.FileWriter
import java.io.IOException
import org.apache.poi.ss.usermodel.CellType
import org.apache.poi.ss.usermodel.Workbook
import org.apache.poi.xssf.usermodel.XSSFWorkbook
import java.io.FileInputStream

val customRedColor = Color(0xFFB70D1B)

@Composable
fun homePage(
    onGlossaireClick: () -> Unit,
    onCodeAVerifierClick: () -> Unit
) {

    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally

    ) {
        Text("Quali'Nomme", style = MaterialTheme.typography.h3)

        var isCompareClicked by remember { mutableStateOf(false) }

        Spacer(modifier = Modifier.height(16.dp))

        Row(
            horizontalArrangement = Arrangement.spacedBy(16.dp)
        ) {
            Button(
                onClick = onGlossaireClick,
                colors = ButtonDefaults.buttonColors(
                    backgroundColor = customRedColor,
                    contentColor = Color.White
                )
            ) {
                Text("Glossaire")
            }

            Button(
                onClick = onCodeAVerifierClick,
                colors = ButtonDefaults.buttonColors(
                    backgroundColor = customRedColor,
                    contentColor = Color.White
                )
            ) {
                Text("Code à Vérifier")
            }

            Button(
                onClick = {
                    isCompareClicked = true

                },
                colors = ButtonDefaults.buttonColors(
                    backgroundColor = customRedColor,
                    contentColor = Color.White
                )
            ) {
                Text("Comparer")
            }

            if (isCompareClicked) {
                println(mostUsedWordList.keys.toList())
                compareResults(
                    motsGlossaire = chargerDonneesDepuisFichier(),
                    motsCode = mostUsedWordList.keys.toList(),
                    onRetourClick = { isCompareClicked = false }
                )
            }
        }
    }
}



@Composable
fun compareResults(
    motsGlossaire: List<Mot>,
    motsCode: List<String>,
    onRetourClick: () -> Unit,
    navigationIcon: @Composable () -> Unit = { Icon(Icons.Filled.ArrowBack, contentDescription = null) }

) {
    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Top, // Align content at the top
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Text(text = "Le code contient ${comparerMots(motsGlossaire, motsCode)}% des mots du glossaire\"", style = MaterialTheme.typography.h3)
    }

    Column(
        modifier = Modifier
            .fillMaxSize()  // Fills the maximum available width
            .padding(16.dp),
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally

    ) {
        Text(text = "Le code contient ${comparerMots(motsGlossaire, motsCode)}% des mots du glossaire")
    }

    // Bouton retour positionné tout en bas et centré horizontalement
    Column(
        modifier = Modifier.fillMaxSize().padding(20.dp),
        verticalArrangement = Arrangement.Bottom,
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Button(
            onClick = onRetourClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("Retour")
        }
    }

}

fun comparerMots(motsGlossaire: List<Mot>, motsCode: List<String>): Int {
    var motsTrouves = 0
    motsCode.forEach { motCode ->
        motsGlossaire.forEach { mot ->
            if (motCode.equals(mot.nom, ignoreCase = true)) {
                motsTrouves++
            }
        }
    }
    println(motsTrouves*100/motsCode.size)
    return motsTrouves*100/motsCode.size
}

var mostUsedWordList = mutableMapOf<String, Int>()
@Composable
@Preview
fun app() {
    MaterialTheme {
        val currentPage = remember { mutableStateOf("accueil") }



        when (currentPage.value) {
            "accueil" -> {
                homePage(
                    onGlossaireClick = { currentPage.value = "glossaire" },
                    onCodeAVerifierClick = { currentPage.value = "choixLangage" }
                )
            }

            "glossaire" -> {
                glossairePage(
                    onAjouterMotClick = { currentPage.value = "formulaire" },
                    onImporterClick = {
                        selectFile(setOf("csv", "xlsx")) { filePath ->
                            println("Importing file: $filePath")
                            importFile(filePath)
                        }
                    },
                    onExporterClick = {
                        val fileDialog = FileDialog(Frame(), "Save as CSV", FileDialog.SAVE)
                        fileDialog.file = "glossaire_exporte.csv" // Initial file name
                        fileDialog.isVisible = true
                        val selectedFile = fileDialog.file
                        val selectedDirectory = fileDialog.directory
                        if (selectedFile != null) {
                            val csvFilePath = "$selectedDirectory$selectedFile"
                            println("Exporting to: $csvFilePath")
                            exportToCSV(csvFilePath)
                        } else {
                            println("Export command cancelled by user.")
                        }
                    },
                    onRetourClick = { currentPage.value = "accueil" }
                )
            }

            "formulaire" -> {
                formulairePage(onAnnulerClick = { currentPage.value = "glossaire" })
            }

            "choixLangage" -> {
                val pythonExtensions = setOf("py")
                val javaExtensions = setOf("java")
                val jsExtensions = setOf("js", "html")
                choixLangagePage(
                    onRetourClick = { currentPage.value = "accueil" },
                    onPythonClick = {
                        selectFile(pythonExtensions) { filePath ->
                            println("Python file selected: $filePath") // Change by parser functions
                        }
                    },
                    onJavaClick = {
                        selectFile(javaExtensions) { filePath ->
                            println("Java file selected: $filePath") // Change by parser functions
                        }
                    },
                    onJavaScriptClick = {
                        selectFile(jsExtensions) { filePath ->
                            mostUsedWordList = parser(filePath) // Change by parser functions
                        }
                    }
                )
            }
            // ... Ajoutez d'autres cas pour les pages "glossaire," "code a verifier," et "comparer"
        }
    }
}

fun selectFile(extensions: Set<String>, onFileSelected: (String) -> Unit) {
    val fileDialog = FileDialog(Frame(), "Select a file", FileDialog.LOAD)
    fileDialog.isMultipleMode = false // To enable selecting only one file
    fileDialog.file = "*." + extensions.joinToString(";*.")
    fileDialog.isVisible = true


    val selectedFile = fileDialog.file
    val selectedDirectory = fileDialog.directory

    if (selectedFile != null) {
        val filePath = "$selectedDirectory$selectedFile"

        // Vérifier si l'extension est autorisée
        val fileExtension = File(filePath).extension.lowercase()
        if (extensions.contains(fileExtension)) {
            println("Opening: $filePath")
            onFileSelected(filePath)
        } else {
            println("Invalid file extension.")
        }
    } else {
        println("Open command cancelled by user.")
    }
}

fun exportToCSV(csvFilePath: String) {
    val glossary = chargerDonneesDepuisFichier()
    val csvContent = buildString {
        appendLine("Mot;Description;Contexte principal;Contexte 2;Lié à;Synonyme;Antonyme;")
        glossary.forEach { entry ->
            appendLine(
                "${entry.nom};${entry.description};${entry.contextePrincipal};" +
                        "${entry.contexte2};${entry.lieA};${entry.synonyme};${entry.antonyme}"
            )
        }
    }
    try {
        FileWriter(csvFilePath).use { fileWriter ->
            fileWriter.write(csvContent)
        }
    } catch (e: IOException) {
        e.printStackTrace()
    }
}

fun importFile(cheminFichier: String) {
    val fileExtension = File(cheminFichier).extension.lowercase()

    when (fileExtension) {
        "csv" -> {
            importCSVFile(cheminFichier)
        }
        "xlsx" -> {
            importXLSXFile(cheminFichier)
        }
        else -> {
            println("Unsupported file format.")
        }
    }
}

fun importCSVFile(cheminFichier: String) {
    val file = File(cheminFichier).readText(Charsets.UTF_8)

    val lines = file.split("\n")
    val header = lines.first().split(";")
    val dataLines = lines.drop(1)

    dataLines.forEach { line ->
        // Check if the line is not empty or blank
        if (line.isNotBlank()) {
            val values = line.replace("\r", "").replace("\n", "").split(";")

            values.forEach { value ->
                println(value)
            }
            val mot = header.zip(values).toMap().toMutableMap()
            val nouveauMot = Mot(
                nom = mot["Mot"]!!,
                description = mot["Description"]!!,
                contextePrincipal = mot["Contexte principal"]!!,
                contexte2 = mot["Contexte 2"]!!,
                lieA = mot["Lié à"]!!,
                synonyme = mot["Synonyme"]!!,
                antonyme = mot["Antonyme"]!!
            )

            if (mot["Antonyme"] == "\r") {
                mot["Antonyme"] = ""
            }
            if (mot["Mot"] == null || mot["Mot"] == "") {
                return
            }
            if (!verifierChampsRequis(mot)) {
                return
            }
            ajouterMotAuGlossaire(nouveauMot)
        }
    }
}

private fun importXLSXFile(cheminFichier: String) {
    val workbook: Workbook
    try {
        FileInputStream(cheminFichier).use { fileInputStream ->
            workbook = XSSFWorkbook(fileInputStream)
        }

        val sheet = workbook.getSheetAt(0) // Assuming the data is in the first sheet

        val headerRow = sheet.getRow(0)
        val header = mutableListOf<String>()
        for (i in 0 until headerRow.lastCellNum) {
            header.add(headerRow.getCell(i).stringCellValue)
        }

        val dataLines = mutableListOf<Map<String, String>>()

        for (i in 1 until sheet.physicalNumberOfRows) {
            val currentRow = sheet.getRow(i)
            val data = mutableMapOf<String, String>()

            for (j in 0 until currentRow.lastCellNum) {
                val cell = currentRow.getCell(j)
                val cellValue = when (cell.cellType) {
                    CellType.STRING -> cell.stringCellValue
                    CellType.NUMERIC -> cell.numericCellValue.toString()
                    else -> ""
                }
                data[header[j]] = cellValue
            }

            dataLines.add(data)
        }

        dataLines.forEach { line ->
            // Check if the line is not empty or blank
            if (line.isNotEmpty()) {
                // Process each line as needed
                val nouveauMot = Mot(
                    nom = line["Mot"] ?: "",
                    description = line["Description"] ?: "",
                    contextePrincipal = line["Contexte principal"] ?: "",
                    contexte2 = line["Contexte 2"] ?: "",
                    lieA = line["Lié à"] ?: "",
                    synonyme = line["Synonyme"] ?: "",
                    antonyme = line["Antonyme"] ?: ""
                )

                if (nouveauMot.nom.isNotBlank() && verifierChampsRequis(line)) {
                    ajouterMotAuGlossaire(nouveauMot)
                }
            }
        }
    } catch (e: IOException) {
        e.printStackTrace()
    }
}

fun verifierChampsRequis(mot: Map<String, String>): Boolean {
    //Vérifier que les headers sont égales aux champs requis
    val champsRequis = listOf(
        "Mot",
        "Description",
        "Contexte principal",
        "Contexte 2",
        "Lié à",
        "Synonyme",
        "Antonyme"
    )

    val motKeys = mot.keys.map { it.trim() }

    motKeys.forEach { key ->
        if (!champsRequis.contains(key)) {
            println("Le champ $key n'est pas reconnu")
            return false
        }
    }

    return true

}



@Composable
fun choixLangagePage(
    onRetourClick: () -> Unit,
    onPythonClick: () -> Unit,
    onJavaClick: () -> Unit,
    onJavaScriptClick: () -> Unit
) {

    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Top, // Align content at the top
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Text(text = "Choix du langage", style = MaterialTheme.typography.h3)
    }

    Column(
        modifier = Modifier
            .fillMaxSize()  // Fills the maximum available width
            .padding(50.dp),
        // allign the content in the center vertically and horizontally
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally

    ) {
        Button(
            onClick = onPythonClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("Python")
        }

        Button(
            onClick = onJavaClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("Java")
        }

        Button(
            onClick = onJavaScriptClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("JavaScript")
        }
    }

    // Bouton retour positionné tout en bas et centré horizontalement
    Column(
        modifier = Modifier.fillMaxSize().padding(20.dp),
        verticalArrangement = Arrangement.Bottom,
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Button(
            onClick = onRetourClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("Retour")
        }
    }

}

@Composable
fun glossairePage(
    onAjouterMotClick: () -> Unit,
    onImporterClick: () -> Unit,
    onExporterClick: () -> Unit,
    onRetourClick: () -> Unit
) {

    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Top, // Align content at the top
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Text(text = "Glossaire", style = MaterialTheme.typography.h3)
    }

    Column(
        modifier = Modifier
            .fillMaxSize()  // Fills the maximum available width
            .padding(16.dp),
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally

    ) {

        Button(
            onClick = onAjouterMotClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("Ajouter un mot")
        }

        Button(
            onClick = onImporterClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("Importer un fichier CSV ou XLSX")
        }

        Button(
            onClick = onExporterClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("Exporter un fichier CSV")
        }

    }

    // Bouton retour positionné tout en bas et centré horizontalement
    Column(
        modifier = Modifier.fillMaxSize().padding(20.dp),
        verticalArrangement = Arrangement.Bottom,
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Button(
            onClick = onRetourClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("Retour")
        }
    }

}

fun resetFields(
    nom: MutableState<String>,
    description: MutableState<String>,
    contextePrincipal: MutableState<String>,
    contexte2: MutableState<String>,
    lieA: MutableState<String>,
    synonyme: MutableState<String>,
    antonyme: MutableState<String>
) {
    nom.value = ""
    description.value = ""
    contextePrincipal.value = ""
    contexte2.value = ""
    lieA.value = ""
    synonyme.value = ""
    antonyme.value = ""
}

@OptIn(ExperimentalMaterialApi::class)
@Composable
fun formulairePage(onAnnulerClick: () -> Unit) {
    // State to track whether to show the snackbar
    val ChampsObligatoireSnackbarVisibleState = remember { mutableStateOf(false) }
    val ExisteDejaSnackbarVisibleState = remember { mutableStateOf(false) }

    val nom = remember { mutableStateOf("") }
    val description = remember { mutableStateOf("") }
    val contextePrincipal = remember { mutableStateOf("") }
    val contexte2 = remember { mutableStateOf("") }
    val lieA = remember { mutableStateOf("") }
    val synonyme = remember { mutableStateOf("") }
    val antonyme = remember { mutableStateOf("") }

    val openDialog = remember { mutableStateOf(false) }
    if (openDialog.value) {
        AlertDialog(
            onDismissRequest = {
                openDialog.value = false
            },
            title = {
                Text(text = "Confirmation d'ajout de mot")
            },
            text = {
                Text("Le mot a bien été ajouté au glossaire.")
            },
            confirmButton = {
                Button(
                    onClick = {
                        openDialog.value = false
                        resetFields(nom, description, contextePrincipal, contexte2, lieA, synonyme, antonyme)

                    },
                    colors = ButtonDefaults.buttonColors(
                        backgroundColor = customRedColor,
                        contentColor = Color.White
                    )
                ) {
                    Text("Fermer")

                }
            }
        )
    }

    MaterialTheme {
        Column(
            modifier = Modifier.padding(16.dp),
            verticalArrangement = Arrangement.spacedBy(8.dp),
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            Text(text = "Nouveau mot", style = MaterialTheme.typography.h5)

            TextField(
                value = nom.value,
                onValueChange = { nom.value = it },
                label = { Text("Mot *") },
            )

            TextField(
                value = description.value,
                onValueChange = { description.value = it },
                label = { Text("Description") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor,
                    unfocusedIndicatorColor = Color.Gray
                )
            )

            TextField(
                value = contextePrincipal.value,
                onValueChange = { contextePrincipal.value = it },
                label = { Text("Contexte principal *") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor,
                    unfocusedIndicatorColor = Color.Gray
                )
            )

            TextField(
                value = contexte2.value,
                onValueChange = { contexte2.value = it },
                label = { Text("Contexte 2") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor,
                    unfocusedIndicatorColor = Color.Gray
                )
            )

            TextField(
                value = lieA.value,
                onValueChange = { lieA.value = it },
                label = { Text("Lié à") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor,
                    unfocusedIndicatorColor = Color.Gray
                )
            )

            TextField(
                value = synonyme.value,
                onValueChange = { synonyme.value = it },
                label = { Text("Synonyme") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor,
                    unfocusedIndicatorColor = Color.Gray
                )
            )

            TextField(
                value = antonyme.value,
                onValueChange = { antonyme.value = it },
                label = { Text("Antonyme") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor,
                    unfocusedIndicatorColor = Color.Gray
                )
            )

            Row(
                modifier = Modifier.fillMaxWidth(),
                horizontalArrangement = Arrangement.SpaceEvenly
            ) {
                Button(
                    onClick = onAnnulerClick,
                    colors = ButtonDefaults.buttonColors(
                        backgroundColor = customRedColor,
                        contentColor = Color.White
                    )
                ) {
                    Icon(imageVector = Icons.Default.Close, contentDescription = null)
                    Text("Annuler")
                }
                Button(
                    onClick = {
                        // Validation du formulaire
                        if (nom.value.isBlank() || contextePrincipal.value.isBlank()) {
                            // Show the snackbar when validation fails
                            ChampsObligatoireSnackbarVisibleState.value = true
                            return@Button
                        }

                        val listeMots = chargerDonneesDepuisFichier().toMutableList()

                        // Vérifier si le mot existe déjà dans le glossaire
                        if (listeMots.any { it.nom.equals(nom.value, ignoreCase = true) }) {
                            println("Le mot '${nom.value}' existe déjà dans le glossaire. Ajout annulé.")
                            ExisteDejaSnackbarVisibleState.value = true
                            return@Button
                        }

                        val nouveauMot = Mot(
                            nom = nom.value,
                            description = description.value,
                            contextePrincipal = contextePrincipal.value,
                            contexte2 = contexte2.value,
                            lieA = lieA.value,
                            synonyme = synonyme.value,
                            antonyme = antonyme.value
                        )

                        ajouterMotAuGlossaire(nouveauMot)
                        openDialog.value = true

                    },
                    colors = ButtonDefaults.buttonColors(
                        backgroundColor = customRedColor,
                        contentColor = Color.White
                    )
                ) {
                    Icon(imageVector = Icons.Default.Check, contentDescription = null)
                    Text("Valider")
                }
            }

            // Show the snackbar when the state is true
            if (ChampsObligatoireSnackbarVisibleState.value) {
                Snackbar(
                    modifier = Modifier.padding(16.dp),
                    action = {
                        Button(onClick = { ChampsObligatoireSnackbarVisibleState.value = false }) {
                            Text("OK")
                        }
                    }
                ) {
                    Text("Les champs 'Mot' et 'Contexte principal' sont obligatoires.")
                }
            }
            if (ExisteDejaSnackbarVisibleState.value) {
                Snackbar(
                    modifier = Modifier.padding(16.dp),
                    action = {
                        Button(onClick = { ExisteDejaSnackbarVisibleState.value = false }) {
                            Text("OK")
                        }
                    }
                ) {
                    Text("Le mot '${nom.value}' existe déjà dans le glossaire. Ajout annulé.")
                }
            }
        }
    }
}


fun chargerDonneesDepuisFichier(): List<Mot> {
    //if file is empty, return empty list
    if (!File("glossaire.json").exists() || File("glossaire.json").length() == 0L) {
        return emptyList()
    }
    return try {
        val content = File("glossaire.json").readText()
        Json.decodeFromString<List<Mot>>(content)
    } catch (e: IOException) {
        e.printStackTrace()
        emptyList() // Handle the exception as needed
    }
}

private val json = Json { prettyPrint = true }

fun sauvegarderDonneesDansFichier(listeMots: List<Mot>) {
    try {
        val content = json.encodeToString(listeMots)
        File("glossaire.json").writeText(content)
    } catch (e: IOException) {
        e.printStackTrace()
    }
}

fun ajouterMotAuGlossaire(nouveauMot: Mot) {
    val listeMots = chargerDonneesDepuisFichier().toMutableList()

    // Vérifier si le mot existe déjà dans le glossaire
    if (listeMots.any { it.nom.equals(nouveauMot.nom, ignoreCase = true) }) {
        println("Le mot '${nouveauMot.nom}' existe déjà dans le glossaire. Ajout annulé.")
        return
    }

    // Ajouter le nouveau mot seulement s'il n'existe pas déjà
    listeMots.add(nouveauMot)
    sauvegarderDonneesDansFichier(listeMots)

    println("Mot ajouté avec succès : ${nouveauMot.nom}")
}


fun main() = application {
    val state = rememberWindowState(
        placement = WindowPlacement.Floating,
        position = WindowPosition(Alignment.Center),
        isMinimized = false,
        width = 1280.dp,
        height = 720.dp
    )

    Window(
        title = "Quali'Nomme",
        resizable = true,
        state = state,
        icon = painterResource("assets/logo/logo.png"),
        onCloseRequest = ::exitApplication
    ) {
        app()
    }
}
